[PATCH] Find size of SHA1 object without inflating everything.
[git.git] / diff.c
diff --git a/diff.c b/diff.c
index d5881c7..5513632 100644 (file)
--- a/diff.c
+++ b/diff.c
@@ -132,10 +132,16 @@ static void builtin_diff(const char *name_a,
                            diff_arg, input_name_sq[0], input_name_sq[1]);
 
        printf("diff --git a/%s b/%s\n", name_a, name_b);
-       if (!path1[0][0])
+       if (!path1[0][0]) {
                printf("new file mode %s\n", temp[1].mode);
-       else if (!path1[1][0])
+               if (xfrm_msg && xfrm_msg[0])
+                       puts(xfrm_msg);
+       }
+       else if (!path1[1][0]) {
                printf("deleted file mode %s\n", temp[0].mode);
+               if (xfrm_msg && xfrm_msg[0])
+                       puts(xfrm_msg);
+       }
        else {
                if (strcmp(temp[0].mode, temp[1].mode)) {
                        printf("old mode %s\n", temp[0].mode);
@@ -327,7 +333,6 @@ int diff_populate_filespec(struct diff_filespec *s, int size_only)
                close(fd);
        }
        else {
-               /* We cannot do size only for SHA1 blobs */
                char type[20];
                struct sha1_size_cache *e;
 
@@ -337,11 +342,13 @@ int diff_populate_filespec(struct diff_filespec *s, int size_only)
                                s->size = e->size;
                                return 0;
                        }
+                       if (!sha1_file_size(s->sha1, &s->size))
+                               locate_size_cache(s->sha1, s->size);
+               }
+               else {
+                       s->data = read_sha1_file(s->sha1, type, &s->size);
+                       s->should_free = 1;
                }
-               s->data = read_sha1_file(s->sha1, type, &s->size);
-               s->should_free = 1;
-               if (s->data && size_only)
-                       locate_size_cache(s->sha1, s->size);
        }
        return 0;
 }
@@ -733,6 +740,16 @@ static void diff_flush_patch(struct diff_filepair *p)
                        p->one->path, p->two->path);
                msg = msg_;
                break;
+       case 'D': case 'N':
+               if (DIFF_PAIR_BROKEN(p)) {
+                       sprintf(msg_,
+                               "dissimilarity index %d%%",
+                               (int)(0.5 + p->score * 100.0/MAX_SCORE));
+                       msg = msg_;
+               }
+               else
+                       msg = NULL;
+               break;
        default:
                msg = NULL;
        }
@@ -838,12 +855,14 @@ static void diff_resolve_rename_copy(void)
                else if (memcmp(p->one->sha1, p->two->sha1, 20) ||
                         p->one->mode != p->two->mode)
                        p->status = 'M';
-               else
-                       /* this is a "no-change" entry.
-                        * should not happen anymore.
-                        * p->status = 'X';
+               else {
+                       /* This is a "no-change" entry and should not
+                        * happen anymore, but prepare for broken callers.
                         */
-                       die("internal error in diffcore: unmodified entry remains");
+                       error("feeding unmodified %s to diffcore",
+                             p->one->path);
+                       p->status = 'X';
+               }
        }
        diff_debug_queue("resolve-rename-copy done", q);
 }